/* ccgo: igs/conn.cc
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "conn.hh"
#include "../settings.hh"
#include <iostream>
extern "C" {
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/wait.h>
#include <unistd.h>
}

using namespace igs;
using namespace std;

void Conn::process_line()
{
	std::cerr << "input line: " << line_buffer << endl;
}

void Conn::conn_lost()
{
}

Conn::Conn() :
	conn_ok(false)
{
// 	std::cerr << "new Conn" << std::endl;
}

Conn::~Conn()
{
	if (child_pid != 0) {
		grab_line_connection.disconnect();
		time_up_connection.disconnect();
		kill(child_pid, SIGHUP);
		int s;
		waitpid(child_pid, & s, 0);
		child_pid = 0;
	}
}

bool Conn::open_conn(const std::string & n)
{
	conn_ok = false;
	int c2p[2];
	int p2c[2];
	pipe(c2p);
	pipe(p2c);
	child_pid = fork();
	if (child_pid == -1) return false;
	if (child_pid == 0) {
		close(0); // close cin
		dup(p2c[0]); // from parent
		close(1); // close cout
		dup(c2p[1]); // to parent
		close(p2c[0]);
		close(p2c[1]);
		close(c2p[0]);
		close(c2p[1]);
		close(2); // close cerr
		char * shell = "/bin/bash";
		std::string c = std::string ("exec ") + n;
		execl(shell, shell, "-c", c.c_str(), 0);
		std::cerr << "Fail to run telnet!" << std::endl;
		exit(EXIT_FAILURE);
	}
	close(p2c[0]);
	to_child = p2c[1];
	from_child = c2p[0];
	close(c2p[1]);
	// set nonblock on reading child
	fcntl(from_child, F_SETFL, O_NONBLOCK);
	grab_line_connection = Glib::signal_io().connect(SigC::slot(* this, & Conn::grab_line), from_child, Glib::IO_IN | Glib::IO_HUP);
	time_up_connection = Glib::signal_timeout().connect(SigC::slot(* this, & Conn::time_up), 100);
	conn_ok = true;
	no_wait = true;
	reading_buffer = false;
	return true;
}

void Conn::close_conn()
{
	// just kill the child
	if (child_pid) {
		kill(child_pid, SIGHUP);
	}
}

Conn::operator bool()
{
	return conn_ok;
}

void Conn::send_line(const std::string & s)
{
	if (go::settings.dump_igs) std::cout << s << std::endl;
	write(to_child, s.c_str(), s.length());
	write(to_child, "\n", 1);
}

bool Conn::grab_line(Glib::IOCondition)
{
	/*
	  should grab the text line by line...
	 */
	if (! child_pid) return false;
	int s;
	// Check if child is dead
	if (waitpid(child_pid, & s, WNOHANG) != 0) {
		grab_line_connection.disconnect();
		time_up_connection.disconnect();
		child_pid = 0;
		conn_ok = false;
		conn_lost(); // <- this will kill myself, (somehow, it segfaults if we use a signal to do it)
		return false;
	}

	if (! reading_buffer) { // must be true!
		// connect to read_buffer
		grab_line_connection = Glib::signal_idle().connect(SigC::slot(* this, & Conn::read_buffer));
		reading_buffer = true;
	}

	return false;
}

bool Conn::read_buffer()
{
	char b[256];
	int s = 0;
	s = read(from_child, b, 128); // only process 128 chars at a time, this slows the processing but makes the UI more responsive.
	if (s > 0) {
		idle_time = 0;
		char * sb = b;
		int p = 0;
		int i;
		for (i = 0; p < s; i ++) {
			if (sb[i] == '\n') { // break it at new line
				line_buffer.append(sb, i);
				process_line();
				line_buffer = "";
				sb += i + 1; // move the start forward
				i = - 1;
			} else if (iscntrl(sb[i])) sb[i] = ' '; // replace control by space
			p ++;
		}
		line_buffer.append(sb, i); // put the rest in the line buffer
		return true;
	}
	if (no_wait && line_buffer.length() >= 3) {
		std::string e = line_buffer.substr(line_buffer.length() - 3);
		if (e == "#> " || e == "n: " || e == "d: ") { // do not wait for newline for these prompts
// 			std::cerr << "no wait prompt" << std::endl;
			process_line();
			line_buffer = "";
		}
	}
	reading_buffer = false;

	// reconnect to grab_line
	grab_line_connection = Glib::signal_io().connect(SigC::slot(* this, & Conn::grab_line), from_child, Glib::IO_IN | Glib::IO_HUP);
	return false;
}

bool Conn::time_up()
{
	if (idle_time < 30000) {
		idle_time += 100;
		if (idle_time == 3000 && line_buffer.length()) {
			std::cerr << "timeout empty buffer" << std::endl;
			process_line();
			line_buffer = "";
		}
	}
	return true;
}

void Conn::set_no_wait(bool b)
{
	no_wait = b;
}
